import { join } from 'path';
import { superjson, getProjectRoot, getConfig } from '@midwayjs/hooks-core';
import { createHttpRequest, close, createFunctionApp as createFunctionApp$1, createApp as createApp$1 } from '@midwayjs/mock';

function SuperJSONPlugin(enableSuperjson) {
  if (!enableSuperjson) {
    return () => {};
  }

  return req => {
    req.on('response', res => {
      if (res.body && res.type.includes('application/json')) {
        res.body = superjson.deserialize(res.body);
      }
    });
  };
}

async function createAppImplementation(options, isFunction) {
  const root = getProjectRoot(global.testPath);
  const config = getConfig(root);
  const cwd = process.cwd();
  process.chdir(root);
  const creator = isFunction ? createFunctionApp$1 : createApp$1;
  const app = await creator(root, Object.assign({
    baseDir: join(root, config.source)
  }, options));
  process.chdir(cwd);
  return new HooksApplication(app, config);
}

async function createApp(options) {
  return createAppImplementation(options, false);
}
async function createFunctionApp(options) {
  return createAppImplementation(options, true);
}
class HooksApplication {
  constructor(app, config) {
    this.app = void 0;
    this.config = void 0;
    this.app = app;
    this.config = config;
  }

  async runFunction(fn, ...args) {
    const response = await this.request(fn, ...args);

    if (!response.ok) {
      throw response.body;
    }

    if (response.type === 'application/json') {
      return response.body;
    }

    return response.text;
  }

  request(fn, ...args) {
    const supertest = createHttpRequest(this.app);

    if (args.length === 0) {
      return supertest.get(fn._param.url).use(SuperJSONPlugin(this.config.superjson));
    }

    return supertest.post(fn._param.url).use(SuperJSONPlugin(this.config.superjson)).send({
      args
    }).set('Content-Type', 'application/json');
  }

  async close() {
    await close(this.app);
  }

}

export { HooksApplication, createApp, createFunctionApp };
